﻿using System;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Runtime.CompilerServices;

namespace MvcApplication.Models
{
  public sealed class Person : INotifyPropertyChanged
  {
    private bool _isChanged;
    public Boolean IsChanged
    {
      get { return _isChanged; }
      set
      {
        _isChanged = value;
        OnPropertyChanged();
      }
    }

    private string _name;

    [Required]
    public string Name
    {
      get
      {
        return _name;
      }
      set
      {
        _name = value;
        OnPropertyChanged();
      }
    }

    private string _surname;

    [Required]
    public string Surname
    {
      get
      {
        return _surname;
      }
      set
      {
        _surname = value;
        OnPropertyChanged();
      }
    }

    private string _address;
    public string Address
    {
      get
      {
        return _address;
      }
      set
      {
        _address = value;
        OnPropertyChanged();
      }
    }

    private DateTime? _dateOfBirth;
    public DateTime? DateOfBirth
    {
      get
      {
        return _dateOfBirth;
      }
      set
      {
        _dateOfBirth = value;
        OnPropertyChanged();
      }
    }

    public Person()
    {
      DateOfBirth = null;
    }

    public bool IsValid()
    {
      return !String.IsNullOrEmpty(Name) && !String.IsNullOrEmpty(Surname)
        && !String.IsNullOrEmpty(Address);
    }

    public event PropertyChangedEventHandler PropertyChanged;

    private void OnPropertyChanged([CallerMemberName] string propertyName = null)
    {
      var handler = PropertyChanged;
      if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
    }
  }
}
