﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Windows.Controls;

namespace WpfModel.Validation
{
  public sealed class ValidationResultCollection : INotifyCollectionChanged, INotifyPropertyChanged, IEnumerable<ValidationResult>
  {
    private readonly Dictionary<String, ValidationResultBase> _results = new Dictionary<String, ValidationResultBase>();

    public Boolean IsValid
    {
      get { return _results.Values.All(i => i.IsValid); }
    }

    public Boolean IsInvalid
    {
      get { return !IsValid; }
    }
    public String ErrorContent
    {
      get
      {
        if (this.Any())
        {
          return _results.Values.First().ErrorContent.ToString();
        }

        return null;
      }
    }

    public void Add(String key, ValidationResultBase validationResult)
    {
      _results[key] = validationResult;
      OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Add, validationResult));
      OnPropertyChanged("IsValid");
      OnPropertyChanged("IsInvalid");
      OnPropertyChanged("ErrorContent");
    }

    public void Remove(String key)
    {
      if (_results.ContainsKey(key))
      {
        var validationResult = _results[key];
        _results.Remove(key);
        OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Remove, validationResult));
        OnPropertyChanged("IsValid");
        OnPropertyChanged("IsInvalid");
        OnPropertyChanged("ErrorContent");
      }
    }

    public void Clear()
    {
      _results.Clear();
      OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
      OnPropertyChanged("IsValid");
      OnPropertyChanged("IsInvalid");
    }

    public IEnumerator<ValidationResult> GetEnumerator()
    {
      return _results.Select(i => i.Value).GetEnumerator();
    }

    IEnumerator IEnumerable.GetEnumerator()
    {
      return GetEnumerator();
    }

    public event NotifyCollectionChangedEventHandler CollectionChanged;

    private void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
    {
      if (CollectionChanged == null)
      {
        return;
      }

      CollectionChanged(this, e);
    }

    public event PropertyChangedEventHandler PropertyChanged;

    private void OnPropertyChanged([CallerMemberName] string propertyName = null)
    {
      PropertyChangedEventHandler handler = PropertyChanged;
      if (handler != null)
      {
        handler(this, new PropertyChangedEventArgs(propertyName));
      }
    }
  }
}
